import sys
max_am = int(sys.argv[1])
prefix = str(sys.argv[2])

file = open(prefix + "/qlist.txt", "w")
file.write(prefix + "/generated/ecpint_gen.cpp\n")
for j in range(max_am+1):
    for i in range(j+1):
        for k in range(max_am+1):
            if j == i == k == max_am:
                file.write(prefix + "/generated/Q" + str(i) + str(j) + str(k) + ".cpp") 
            else:
                file.write(prefix + "/generated/Q" + str(i) + str(j) + str(k) + ".cpp\n")
file.close()



from lxml import etree

angular_shells = ["s", "p", "d", "f", "g", "h", "i"]

class Shell:
    def __init__(self, lval = 0, nexp = 0, nbfs = 0):
        self.lval = lval
        self.powers = []
        self.nexp = nexp
        self.exps = []
        self.contr = []

class Atom:
    def __init__(self, name="X", nshells = 0):
        self.name = name
        self.ncore = 0
        self.maxl = 0
        self.nshells = nshells
        self.shells = []
        
def parse_ecp(file):
    atoms = []
    atomnames = {}
    lines = file.readlines()
    nlines = len(lines)
    linenumber = 0
    atomnumber = 0
    
    while linenumber < nlines:
        line = lines[linenumber].strip()
        tokens = line.split(',')
        for token in tokens: 
            token = token.replace(' ', '')
            
        if (len(tokens) > 2):
            if (tokens[0].lower() == "ecp"):
                atom_name = tokens[1].lower()
                new_atom = Atom(name=atom_name)

                new_atom.ncore = int(tokens[2])
                new_atom.maxl = int(tokens[3])
                
                for i in range(new_atom.maxl+1):
                    linenumber += 1
                    line = lines[linenumber].strip()
                    
                    tokens = line.split(';')
                    for token in tokens:
                        token = token.replace(' ', '')
                        
                    if(len(tokens) > 1):
                        nprims = int(tokens[0])
                        l = i-1
                        if (i==0):
                            l = new_atom.maxl
                        new_shell = Shell(lval=l, nexp=nprims, nbfs=1)
                        
                        for token in tokens[1:]:
                            subtokens = token.split(',')
                            if (len(subtokens) == 3):
                                new_shell.powers.append(subtokens[0])
                                new_shell.exps.append(subtokens[1])
                                new_shell.contr.append(subtokens[2])
                        
                        new_atom.shells.append(new_shell)
                
                atoms.append(new_atom)
                
        linenumber += 1
    
    return atoms

def write_ecp_basis(atoms, name):
    filename = "xml/" + name + ".xml"
    with open(filename, 'wb') as new_file:
        root = etree.Element("root", name=name)
        tree = etree.ElementTree(root)
        
        for atom in atoms:
            child = etree.SubElement(root, atom.name, ncore = str(atom.ncore), maxl=str(atom.maxl))
            
            for shell in atom.shells:
                schild = etree.SubElement(child, "Shell", lval=str(shell.lval), nexp=str(shell.nexp))
                
                for i in range(shell.nexp):
                    try:
                        xchild = etree.SubElement(schild, "nxc", n=shell.powers[i], x=shell.exps[i], c=shell.contr[i])
                    except:
                        print("ERROR in " + filename + " (" + name + ") : atom " + atom.name + ", shell type " + shell.lval)
                        print("Expected no. of powers/exps/coeffs:", shell.nexp)
                        print("Actual no. of powers: ", len(shell.powers))
                        print("Actual no. of exps: ", len(shell.exps))
                        print("Actual no. of coeffs: ",  len(shell.contr))
        tree.write(new_file, pretty_print = True)

if __name__ == "__main__":
    import sys
    name = sys.argv[1]
    input_file = open('raw/' + name + '.ecp', 'r')
    atoms = parse_ecp(input_file)
    write_ecp_basis(atoms, name)
    



MAX_UNROL_AM = 4

# DO NOT EDIT BELOW HERE

from sympy import *

class Qijk:
    def __init__(self, Ival = 0, Jval = 0, Kval = 0):
        self.i = Ival
        self.j = Jval
        self.k = Kval
        self.size = 2*self.i + self.j + 1
        self.start = self.k-self.i-self.j
        self.end = self.k+self.i
        self.subq = []
        self.terms = []
        self.bases = []
        self.f = []
        self.ga = []
        self.gb = []
        self.h = []
        for i in range(self.size):
            self.bases.append([])
    
    def print(self):
        print("Q", self.i, self.j, self.k)
        for i in range(len(self.bases)):
            print(i+self.start, ":")
            for j in range(len(self.bases[i])):
                print(self.bases[i][j])
    
    def print_simple(self):
         print("Q", self.i, self.j, self.k)
         for i in range(len(self.bases)):
            print(i+self.start, ":", simplify(self.bases[i]))
            
    def print_fgh(self):
        print("Q", self.i, self.j, self.k)
        for i in range(len(self.f)):
            ix = 2*i+self.start
            if ix > 0:
                print("F", ix, ":", simplify(self.f[i]))
        for i in range(len(self.gb)):
            ix = 2*i + self.start + 1
            if ix > 0:
                print("GB", ix, ":", simplify(self.gb[i]))
        for i in range(len(self.ga)):
            ix = 2*i + self.start + 1
            if ix > 0:
                print("GA", ix, ":", simplify(self.ga[i]))
        for i in range(len(self.h)):
            ix = 2*i+self.start + 2
            if ix > 0:
                print("H", ix, ":", simplify(self.h[i]))
    
    def write_code(self, f):
        print("\t\t\t\t\t\t\t\t\tcase", self.i*10000+self.j*100+self.k, ": {", file=f)

        for i in range(len(self.f)):
            ix = 2*i+self.start
            if ix == 2:
                simp = simplify(self.f[i])
                if simp!= 0:
                    print("\t\t\t\t\t\t\t\t\t\tresult = (", simp, ") * values[0];", file=f)
            elif ix > 2:
                simp = simplify(self.f[i])
                if simp!= 0:
                    print("\t\t\t\t\t\t\t\t\t\tresult += (", simp, ") * values[", ix-2, "];", file=f)
                
        for i in range(len(self.gb)):
            ix = 2*i + self.start + 1
            if ix == 1:
                simp = simplify(self.gb[i])
                if simp!= 0:
                    print("\t\t\t\t\t\t\t\t\t\tresult += (", simp, ") * G1B;", file=f)
            elif ix > 1:
                simp = simplify(self.gb[i])
                if simp!= 0:
                    print("\t\t\t\t\t\t\t\t\t\tresult += (", simp, ") * values[", ix-2, "];", file=f)

        for i in range(len(self.ga)):
            ix = 2*i + self.start + 1
            if ix == 1:
                simp = simplify(self.ga[i])
                if simp != 0:
                    print("\t\t\t\t\t\t\t\t\t\tresult += (", simp, ") * G1A;", file=f)

        for i in range(len(self.h)):
            ix = 2*i+self.start + 2
            if ix == 2:
                simp = simplify(self.h[i])
                if simp!= 0:
                    print("\t\t\t\t\t\t\t\t\t\tresult += (", simp, ") * H2;", file=f)
                    
        print("\t\t\t\t\t\t\t\t\t\tbreak;", file=f)
        print("\t\t\t\t\t\t\t\t\t}", file=f)

    def simplify(self):
        simple_bases = []
        for i in range(len(self.bases)):
            x = Symbol('x')
            y = Symbol('y')
            z = Symbol('z')
            z = 0
            for j in range(len(self.bases[i])):
                z = z + parse(self.bases[i][j])
            simple_bases.append(z)
        self.bases = simple_bases
        
    def sort(self):
        for i in range(len(self.bases)):
            if i % 2 == 0:
                self.f.append(self.bases[i])
            else:
                self.gb.append(self.bases[i])
                
    def eliminate(self):
        x = Symbol('x')
        y = Symbol('y')
        z = Symbol('z')
        p = Symbol('p')
        z = 0
        
        if self.start < 1:
            if self.end < 1:
                self.bases.append(z)
            if self.end < 2:
                self.bases.append(z)
                
            w = Symbol('w')
            
            N = self.start
            ix = 0
            gaix = -1
            hix  = -1
            
            while (N < 1):
                z = self.bases[ix]
                w = self.bases[ix+2]
                w = w + (2 * p / (N-1))*z
                self.bases[ix+2] = w
            
                w = self.bases[ix+1]
                w = w - (2 * y / (N-1))*z
                self.bases[ix+1] = w
                
                if ix % 2 == 0:
                    if gaix > -1:
                        w = self.ga[gaix]
                        w = w - (2*x / (N-1))*z
                        self.ga[gaix] = w
                    else:
                        w = -(2*x / (N-1))*z
                        self.ga.append(w)
                        gaix += 1
                        
                    if hix > -1:
                        z = self.h[hix]
                        w = (2 * p / (N-1))*z
                        self.h.append(w)
                        hix += 1
                        
                        w = self.ga[gaix]
                        w = w - (2 * y / (N-1))*z
                        self.ga[gaix] = w
                        
                        w = self.bases[ix+1]
                        w = w - (2 * x / (N-1))*z
                        self.bases[ix+1] = w
                else:
                    if hix > -1:
                        w = self.h[hix]
                        w = w - (2*x / (N-1))*z
                        self.h[hix] = w
                    else:
                        w = - (2*x / (N-1))*z
                        self.h.append(w)
                        hix += 1
                    
                    if gaix > -1:
                        z = self.ga[gaix]
                        w = (2 * p /(N-1)) * z
                        self.ga.append(w)
                        gaix += 1
                        
                        w = self.h[hix]
                        w = w - (2 * y / (N-1))*z
                        self.h[hix] = w
                        
                        w = self.bases[ix+1]
                        w = w - (2*x / (N-1))*z
                        self.bases[ix+1] = w
                    
                N += 1
                ix += 1

def parse(term):
    x = Symbol('x')
    y = Symbol('y')
    p = Symbol('p')
    bits = term.split(',')
    z = Symbol('z')
    z = 1
    for bit in bits:
        bi = bit[:2]
        if bi == "mu":
            ix = 2
            i = 0
            j = 0
            k = 0

            I = bit[ix]
            if I == "-":
                ix += 1
                i = -int(bit[ix])
            else:
                i = int(I)
                
            ix += 1
            J = bit[ix]
            if J == "-":
                ix += 1
                j = -int(bit[ix])
            else:
                j = int(J)

            ix += 1
            K = bit[ix]
            if K == "-":
                ix += 1
                k = -int(bit[ix])
            else:
                k = int(K)

            z = z * (2 + j - i - k)/(2*x)
        elif bi == "nu":
            z = z * (-y/x)
        elif bi == "xi":
            z = z * p/x
        elif bi == "rh":
            j = int(bit[3])
            z = z * (1 - 2*j)/(2*y)
        elif bi == "om":
            z = z * -1 / (2*y)
    return z
            
def unrol(q):
    if (q.i == 0 and q.j == 0):
        return
    elif (q.i > 0):
        q1 = Qijk(Ival = q.i-1, Jval = q.j, Kval = q.k-1)
        q.subq.append(q1)
        q.terms.append("mu" + str(q.i) + str(q.j) + str(q.k))

        q2 = Qijk(Ival = q.i-1, Jval = q.j-1, Kval = q.k)
        q.subq.append(q2)
        q.terms.append("nu")
    
        q3 = Qijk(Ival = q.i-1, Jval = q.j, Kval = q.k+1)
        q.subq.append(q3)
        q.terms.append("xi")
    elif(q.j > 1):
        q1 = Qijk(Ival = 0, Jval = q.j-2, Kval = q.k)
        q.subq.append(q1)
        q.terms.append("sigma")
        
        q2 = Qijk(Ival = 0, Jval = q.j-1, Kval = q.k-1)
        q.subq.append(q2)
        q.terms.append("rho" + str(q.j))
    else:
        q1 = Qijk(Ival = 0, Jval = 0, Kval = q.k)
        q.subq.append(q1)
        q.terms.append("ups")
        
        q2 = Qijk(Ival = 0, Jval = 0, Kval = q.k-1)
        q.subq.append(q2)
        q.terms.append("om")

    for i in range(len(q.subq)):
        unrol(q.subq[i])
    
    return

def collect(q, Q, term):
    if (q.i == 0 and q.j == 0):
        Q.bases[q.k-Q.start].append(term)
    else:
        for i in range(len(q.subq)):
            collect(q.subq[i], Q, term + q.terms[i] + ",")

def algebraic_unrol(i, j, k):
    q = Qijk(Ival = i, Jval = j, Kval = k)
    unrol(q)
    collect(q, q, "")
    q.simplify()
    return q


f = open('radial_gen.part2', 'w')
print("", file=f)
for j in range(MAX_UNROL_AM+1):
    for i in range(j+1):
        for k in range(1, 3*MAX_UNROL_AM+1-i-j):
            if (i + j + k) % 2 == 0: 
                q = algebraic_unrol(i, j, k)
                q.eliminate()
                q.sort()
                q.write_code(f)
                print("", file=f)
